@@ -19,6 +19,7 @@ from rasa.nlu.test import determine_token_labels
 from rasa.nlu.tokenizers.tokenizer import Token
 from rasa.nlu.classifiers import LABEL_RANKING_LENGTH
 from rasa.utils import train_utils
+from rasa.utils.common import raise_warning
 from rasa.utils.tensorflow import layers
 from rasa.utils.tensorflow.transformer import TransformerEncoder
 from rasa.utils.tensorflow.models import RasaModel
@@ -56,6 +57,7 @@ from rasa.utils.tensorflow.constants import (
     SPARSE_INPUT_DROPOUT,
     MASKED_LM,
     ENTITY_RECOGNITION,
+    TENSORBOARD_LOG_DIR,
     INTENT_CLASSIFICATION,
     EVAL_NUM_EXAMPLES,
     EVAL_NUM_EPOCHS,
@@ -77,6 +79,7 @@ from rasa.utils.tensorflow.constants import (
     SOFTMAX,
     AUTO,
     BALANCED,
+    TENSORBOARD_LOG_LEVEL,
 )
 
 
@@ -125,7 +128,7 @@ class DIETClassifier(IntentClassifier, EntityExtractor):
         NUM_HEADS: 4,
         # If 'True' use key relative embeddings in attention
         KEY_RELATIVE_ATTENTION: False,
-        # If 'True' use key relative embeddings in attention
+        # If 'True' use value relative embeddings in attention
         VALUE_RELATIVE_ATTENTION: False,
         # Max position for relative embeddings
         MAX_RELATIVE_POSITION: None,
@@ -169,13 +172,15 @@ class DIETClassifier(IntentClassifier, EntityExtractor):
         # If 'True' the algorithm only minimizes maximum similarity over
         # incorrect intent labels, used only if 'loss_type' is set to 'margin'.
         USE_MAX_NEG_SIM: True,
-        # Scale loss inverse proportionally to confidence of correct prediction
+        # If 'True' scale loss inverse proportionally to the confidence
+        # of the correct prediction
         SCALE_LOSS: True,
         # ## Regularization parameters
         # The scale of regularization
         REGULARIZATION_CONSTANT: 0.002,
         # The scale of how important is to minimize the maximum similarity
-        # between embeddings of different labels.
+        # between embeddings of different labels,
+        # used only if 'loss_type' is set to 'margin'.
         NEGATIVE_MARGIN_SCALE: 0.8,
         # Dropout rate for encoder
         DROP_RATE: 0.2,
@@ -205,6 +210,13 @@ class DIETClassifier(IntentClassifier, EntityExtractor):
         # examples per entity are required.
         # Rule of thumb: you should have more than 100 examples per entity.
         BILOU_FLAG: True,
+        # If you want to use tensorboard to visualize training and validation metrics,
+        # set this option to a valid output directory.
+        TENSORBOARD_LOG_DIR: None,
+        # Define when training metrics for tensorboard should be logged.
+        # Either after every epoch or for every training step.
+        # Valid values: 'epoch' and 'minibatch'
+        TENSORBOARD_LOG_LEVEL: "epoch",
     }
 
     # init helpers
@@ -266,8 +278,8 @@ class DIETClassifier(IntentClassifier, EntityExtractor):
         """Declare instance variables with default values."""
 
         if component_config is not None and EPOCHS not in component_config:
-            logger.warning(
-                f"Please configure the number of '{EPOCHS}' in your configuration file for '{self.__class__.name}'."
+            raise_warning(
+                f"Please configure the number of '{EPOCHS}' in your configuration file."
                 f" We will change the default value of '{EPOCHS}' in the future to 1. "
             )
 
@@ -625,7 +637,7 @@ class DIETClassifier(IntentClassifier, EntityExtractor):
                 return
 
         # keep one example for persisting and loading
-        self.data_example = self.data_example = model_data.first_data_example()
+        self.data_example = model_data.first_data_example()
 
         self.model = self.model_class()(
             data_signature=model_data.get_signature(),
@@ -647,8 +659,8 @@ class DIETClassifier(IntentClassifier, EntityExtractor):
     def _predict(self, message: Message) -> Optional[Dict[Text, tf.Tensor]]:
         if self.model is None:
             logger.debug(
-                f"There is no trained model for '{self.__class__.name}': component is either not trained or "
-                f"didn't receive enough training data."
+                "There is no trained model: component is either not trained or "
+                "didn't receive enough training data."
             )
             return
 
@@ -826,7 +838,7 @@ class DIETClassifier(IntentClassifier, EntityExtractor):
 
         if not model_dir or not meta.get("file"):
             logger.debug(
-                f"Failed to load '{cls.name}'. "
+                f"Failed to load model. "
                 f"Maybe the path '{os.path.abspath(model_dir)}' doesn't exist?"
             )
             return cls(component_config=meta)
@@ -935,7 +947,12 @@ class DIET(RasaModel):
         index_tag_id_mapping: Optional[Dict[int, Text]],
         config: Dict[Text, Any],
     ) -> None:
-        super().__init__(name="DIET", random_seed=config[RANDOM_SEED])
+        super().__init__(
+            name="DIET",
+            random_seed=config[RANDOM_SEED],
+            tensorboard_log_dir=config[TENSORBOARD_LOG_DIR],
+            tensorboard_log_level=config[TENSORBOARD_LOG_LEVEL],
+        )
 
         self.config = config
 
@@ -1080,7 +1097,7 @@ class DIET(RasaModel):
             self.config[SIMILARITY_TYPE],
         )
 
-    def _prepare_dot_product_loss(self, name: Text) -> None:
+    def _prepare_dot_product_loss(self, name: Text, scale_loss: bool) -> None:
         self._tf_layers[f"loss.{name}"] = layers.DotProductLoss(
             self.config[NUM_NEG],
             self.config[LOSS_TYPE],
@@ -1088,7 +1105,7 @@ class DIET(RasaModel):
             self.config[MAX_NEG_SIM],
             self.config[USE_MAX_NEG_SIM],
             self.config[NEGATIVE_MARGIN_SCALE],
-            self.config[SCALE_LOSS],
+            scale_loss,
             # set to 1 to get deterministic behaviour
             parallel_iterations=1 if self.random_seed is not None else 1000,
         )
@@ -1122,20 +1139,24 @@ class DIET(RasaModel):
         self._prepare_embed_layers(f"{name}_lm_mask")
         self._prepare_embed_layers(f"{name}_golden_token")
 
-        self._prepare_dot_product_loss(f"{name}_mask")
+        # mask loss is additional loss
+        # set scaling to False, so that it doesn't overpower other losses
+        self._prepare_dot_product_loss(f"{name}_mask", scale_loss=False)
 
     def _prepare_label_classification_layers(self) -> None:
         self._prepare_embed_layers(TEXT)
         self._prepare_embed_layers(LABEL)
 
-        self._prepare_dot_product_loss(LABEL)
+        self._prepare_dot_product_loss(LABEL, self.config[SCALE_LOSS])
 
     def _prepare_entity_recognition_layers(self) -> None:
         self._tf_layers["embed.logits"] = layers.Embed(
             self._num_tags, self.config[REGULARIZATION_CONSTANT], "logits"
         )
         self._tf_layers["crf"] = layers.CRF(
-            self._num_tags, self.config[REGULARIZATION_CONSTANT]
+            self._num_tags,
+            self.config[REGULARIZATION_CONSTANT],
+            self.config[SCALE_LOSS],
         )
         self._tf_layers["crf_f1_score"] = tfa.metrics.F1Score(
             num_classes=self._num_tags - 1,  # `0` prediction is not a prediction
@@ -1214,16 +1235,18 @@ class DIET(RasaModel):
             features, mask, name, sparse_dropout=self.config[SPARSE_INPUT_DROPOUT]
         )
 
-        x = self._tf_layers[f"ffnn.{name}"](inputs, self._training)
+        inputs = self._tf_layers[f"ffnn.{name}"](inputs, self._training)
 
         if masked_lm_loss:
-            x, lm_mask_bool = self._tf_layers[f"{name}_input_mask"](
-                x, mask, self._training
+            inputs, lm_mask_bool = self._tf_layers[f"{name}_input_mask"](
+                inputs, mask, self._training
             )
         else:
             lm_mask_bool = None
 
-        outputs = self._tf_layers[f"{name}_transformer"](x, 1 - mask, self._training)
+        outputs = self._tf_layers[f"{name}_transformer"](
+            inputs, 1 - mask, self._training
+        )
         outputs = tfa.activations.gelu(outputs)
 
         return outputs, inputs, seq_ids, lm_mask_bool
